/* © brix cross media AG, http://www.brix.ch - It is strictly forbidden to change, reuse or copy the source code of this document. */
;(function ( $, window, document, undefined ) {
    var pluginName = "anuraBasket",
        defaults = {
            server: 'http://localhost/anura',
            ext: '.do?', // extension of server pages (.php, .do)
            locale: '', // 2-char language code, blank for default
            force_zip: false, // force even single asset downloads to be delivered in a ZIP-file
            add: null, // IDs to add (int or array of int)
            remove: null, // IDs to remove (int or array of int)
            action: null, // one of: show, hide, toggle
            cookies: true, // use cookies to persist the basket (requires jquery.cookies.js)
            send: false, // add a mailto-link (ID of the download format to use or true for the first)
            clear_after_download: true, // empties the basket after the download has started
            legacy_download_syntax: false, // use foo=1&foo=2 instead of foo=1,2 (for older backends)
            terms_and_conditions: null, // optional, {title: 'T&C', message: 'Lorem Ipsum', confirm: 'I agree'}, localize with {de: 'Ich bin einverstanden', en: 'I agree'}
            close_icon: '&#215;'
        };
    function Plugin( element, options ) {
        this.element = this.container = $(element);
        this.options = $.extend( {}, defaults, options );
        this._defaults = defaults;
        this._name = pluginName;
        this.strings = {};
        var _this = this;
        this.getJSON(this.options.server+'/asset'+this.options.ext+'callback=?&string='+
                ['MENUITEMID_DOWNLOAD', 'search.cleanUp', 'MENUITEMID_SEND', 'MENUITEMID_MAILTO'].join('&string='),
                {locale: this.options.locale}, 'anuraBasketPopupStrings', function(data) {
                    if (data) _this.strings = data;
                    _this.init();
                });
    }
    Plugin.prototype = {
        init: function() {
            var _this = this;
            this.options.counter = this.element.find('.counter').first();
            if (!this.options.counter.length) this.options.counter = this.element.find('span').first();
            if (this.element.data('assets') == null) {
                this.element.data('assets', []);
                this.element.on('click', function() {_this.togglePopup()});
                var overlay = $('<div id="anura-basket-overlay"/>').appendTo($('body'))
                    .on('click', function(e) {if (e.target.id == 'anura-basket-overlay') _this.togglePopup();});
                var popup = $('<div />').addClass('anura-basket-popup anura-locale-'+this.options.locale).appendTo(overlay);
                var titlebar = $('<div />').addClass('anura-basket-popup-titlebar').appendTo(popup);
                $('<span />').html(this.options.close_icon).appendTo(titlebar).on('click', function() {_this.togglePopup();});
                $('<span />').text((_this.strings['search.cleanUp'] || 'clear').toLowerCase()).addClass('anura-basket-reset').appendTo(popup).on('click', function() {_this.clear()});
                this.requestDownloadFormats(function() {
                    var menu_holder = $('<div />').addClass('anura-basket-download').appendTo(popup);
                    if (_this.options.terms_and_conditions) menu_holder = _this.addTerms(menu_holder);
                    $('<h3> />').addClass('anura-basket-download-title').text((_this.strings['MENUITEMID_DOWNLOAD'] || 'Download')+':').appendTo(menu_holder);
                    $.each(_this.download_formats, function(index, format) {
                        $('<div />').addClass('anura-basket-download-holder').text(format.name).attr('title', format.name)
                            .css('background-image', 'url(\''+_this.options.server+'/'+format.icon+'\')')
                            .on('click', function() {
                                if ($('#anura-basket-tc-confirm-cb').length && !$('#anura-basket-tc-confirm-cb').is(':checked')) return;
                                var opened = window.open(_this.getDownloadUrl(_this.element.data('assets'), format.id, format.url), 'blank_'+new Date().getTime());
                                if (opened && _this.options.clear_after_download) _this.clear();
                        }).appendTo(menu_holder);
                    });
                    if (_this.options.send) {
                        $('<h3> />').addClass('anura-basket-download-send').text((_this.strings['MENUITEMID_SEND'] || 'Send')+':').appendTo(menu_holder);
                        $('<div />').addClass('anura-basket-download-holder mailto').text(_this.strings['MENUITEMID_MAILTO'])
                            .attr('title', _this.strings['MENUITEMID_MAILTO']).on('click', function() {
                                if ($('#anura-basket-tc-confirm-cb').length && !$('#anura-basket-tc-confirm-cb').is(':checked')) return;
                                var link = _this.getDownloadUrl(_this.element.data('assets'), _this.options.send, null);
                                window.location = 'mailto:?body='+encodeURIComponent(link);
                            }).appendTo(menu_holder);
                    }
                    _this.element.trigger('anura-loaded', [_this.options]);
                });
                $('<div />').addClass('anura-basket-thumbs').appendTo(popup);
                if (_this.options.cookies && $.cookie && $.cookie('anura-basket'))
                    $.each(JSON.parse($.cookie('anura-basket')), function(intex, item) {_this.add(item)});
            }
            if (this.options.add) {
                if ($.isArray(this.options.add)) {
                    $.each(this.options.add, function() {_this.add(this)});
                } else {
                    this.add(this.options.add);
                }
                this.options.add = null;
            }
            if (this.options.remove) {
                if ($.isArray(this.options.remove)) {
                    $.each(this.options.remove, function() {_this.remove(this)});
                } else {
                    this.remove(this.options.remove);
                }
                this.options.remove = null;
            }
            if (this.options.action) {
                switch (this.options.action) {
                    case 'show': $('#anura-basket-overlay').show(); break;
                    case 'hide': $('#anura-basket-overlay').hide(); break;
                    case 'toggle': this.togglePopup(); break;
                }
            }
        },
        getDownloadUrl: function(assets, format, url) {
			assets.sort(); // so the order doesn't confuse the cache
            return this.options.server+'/'+(url?url:('asset'+this.options.ext))+
                'download='+assets.join(this.options.legacy_download_syntax?'&download=':',')+
                '&format='+format+(this.options.force_zip?'&forcezip=true':'');
        },
        add: function(asset) {
            if (!asset) return;
            var assets = this.element.data('assets'), _this = this;
            if ($.inArray(asset, assets) !== -1) return; // prevent duplicates
            assets.push(asset);
            if (this.options.counter) this.options.counter.html(assets.length);
            var container = $('<div />').attr('id', 'anura-basket-asset-'+asset).addClass('anura-basket-thumb')
                .appendTo($('#anura-basket-overlay .anura-basket-thumbs')).on('click', function(e) {
                    if ($('#basket'+asset).length) $('#basket'+asset).click();
                    else _this.remove(asset);
                });
            $('<img />').attr('src', _this.options.server+'/asset'+_this.options.ext+'thumb='+asset+'&img=a.jpg')
                /* IE */.removeAttr('width').removeAttr('height')/* /IE */.appendTo(container);
            $('<div />').addClass('anura-basket-thumb-delete hover').appendTo(container);
            if (this.options.cookies && $.cookie) $.cookie('anura-basket', JSON.stringify(assets), {path: '/'});
        },
        remove: function(asset) {
            if (!asset) return;
            var assets = this.element.data('assets');
            assets.splice($.inArray(asset, assets),1);
            $('#anura-basket-asset-'+asset).remove();
            if (this.options.counter) this.options.counter.html(assets.length);
            if (assets.length == 0 && $('#anura-basket-overlay').is(':visible')) this.togglePopup();
            if (this.options.cookies && $.cookie) $.cookie('anura-basket', JSON.stringify(assets), {path: '/'});
        },
        clear: function() {
            this.element.data('assets', []);
            if (this.options.cookies && $.removeCookie) $.removeCookie('anura-basket', {path: '/'});
            $('#anura-basket-overlay .anura-basket-thumb').remove();
            if (this.options.counter) this.options.counter.html(0); // basket indicator
            $('.anura-add-basket.added').removeClass('added'); // gallery indicators
            this.togglePopup();
        },
        togglePopup: function() {
            var overlay = $('#anura-basket-overlay');
            if (overlay.is(':visible') || !this.element.data('assets').length) {
                overlay.hide();
                $('#anura-basket-tc-confirm-cb').prop('checked', false);
                $('.anura-basket-tc-wrapper').removeClass('anura-tc-confirmed');
            } else {
                overlay.show();
            }
            this.element.trigger('anura-toggled');
        },
        requestDownloadFormats: function(callback) {
            var _this = this;
            var onComplete = function(data) {
                window.anura_download_formats = _this.download_formats = data;
                if (_this.options.send === true) _this.options.send = data[0].id;
                if (callback) callback();
            }
            if (window.anura_download_formats) { // share globally
                onComplete(window.anura_download_formats);
            } else {
                this.getJSON(this.options.server+'/asset'+this.options.ext+'callback=?',
                    {downloadformats: 1, locale: this.options.locale}, 'anuraBasketPopupDlf', onComplete);
            }
        },
        addTerms: function(target) {
            var tc = this.options.terms_and_conditions, loc = this.options.locale;
            $('<h3 />').addClass('anura-basket-tc-title').html((tc.title[loc] ? tc.title[loc] : tc.title)+':').appendTo(target);
            $('<p />').addClass('anura-basket-tc-msg').html(tc.message[loc] ? tc.message[loc] : tc.message).appendTo(target);
            if (tc.confirm) {
                var confirm = $('<p/>').addClass('anura-basket-tc-confirm').appendTo(target);
                $('<input type="checkbox"/>').attr('id', 'anura-basket-tc-confirm-cb').appendTo(confirm).on('change', function() {
                    $('.anura-basket-tc-wrapper').toggleClass('anura-tc-confirmed');
                });
                $('<label />').attr('for', 'anura-basket-tc-confirm-cb').html(' '+(tc.confirm[loc] ? tc.confirm[loc] : tc.confirm)).appendTo(confirm);
                return $('<div />').addClass('anura-basket-tc-wrapper').appendTo(target);
            } else {
                return target;
            }
        },
        getJSON: function(url, data, func_id, func, func_fail) {
            var _this = this;
            if (this.element.attr('id')) func_id = func_id + this.element.attr('id').replace(/\W/g, ''); // for multi-instance
            var on_fail = function(response) {
                _this.container.addClass('anura-error').removeClass('anura-loading');
                if (response) _this.element.data('status', response.status).data('message', response.message);
                _this.element.trigger('anura-error', [response]);
                if (func_fail) func_fail(response);
            }
            window[func_id] = function(response) {
                if (!response || (response.status && response.status != 200)) {
                    on_fail(response);
                } else {
                    _this.version = response.version;
                    _this.container.removeClass('anura-error');
                    _this.element.data('status', null).data('message', null);
                    func(response.data ? response.data : response);
                }
            }
            $.ajax({dataType: "json", url: url, data: data, traditional: true, cache: true, jsonp: true, jsonpCallback: func_id}).fail(on_fail);
        }
    };
    $.fn[pluginName] = function ( options ) {
        return this.each(function (i, item) { // iterate matches
            var instance = $.data(item, "plugin_" + pluginName);
            if (!instance) { // if it has no instance yet, create it
                $.data(item, "plugin_" + pluginName, new Plugin( item, options ));
            } else { // otherwise merge new options with existing ones, empty and re-init
                instance.options = $.extend( {}, instance.options, options );
                instance.init();
            }
        });
    };
})( jQuery, window, document );
