/* © brix cross media AG, http://www.brix.ch - It is strictly forbidden to change, reuse or copy the source code of this document. */
;(function ( $, window, document, undefined ) {
    var pluginName = "anuraBasket",
        defaults = {
            server: 'https://celum.local/anura/first', // REQUIRED, which anura endpoint to use
            ext: '.do?', // extension of server pages (.php, .do)
            locale: '', // ISO 639-1 code, blank for default
            force_zip: false, // force even single asset downloads to be delivered in a ZIP-file
            add: null, // IDs to add (int or array of int)
            remove: null, // IDs to remove (int or array of int)
            action: null, // one of: show, hide, toggle
            cookies: false, // use cookies to persist the basket (requires jquery.cookies.js)
            local_storage: true, // use localStorage to persist the basket (mutually exclusive with the "cookies" option!)
            send: false, // add a mailto-link
            pin: false, // add pin creation link
            require_email: false, // require the downloader to input his mail address (only useful when also handled in CELUM)
            clear_after_download: true, // empties the basket after the download has started
            terms_and_conditions: null, // optional, prevents downloading the basket's content until the user has agreed to some terms and conditions
                                        // {title: 'T&C', message: 'Lorem Ipsum', confirm: 'I agree', show: function() {return true; /*optional callback if terms should be shown, e.g. based on infofields*/}, confirmed: function() {/*callback when user has confirmed terms*/}}
                                        // localize strings with {de: 'Ich bin einverstanden', en: 'I agree'}
            icon_transform: function(icon) {return icon.replace('/big/', '/small/')},
            close_icon: '&#215;', // symbol for the close button
            show_description: true, // show the description of a download format as a separate line (instead of a tooltip)
            max_assets: 0, // limit the amount of assets that can be added to the basket
            infofields: [], // list of information field IDs to load with the download response
            show_failed: false, // show placeholders for assets that couldn't be added to the basket (useful when share=true)
            default_format: null, // set the download format ID that should be selected initially (when available)
            alternative_name: { // alternative name (shown instead of the asset name)
                id: 0, // information field ID
            }
        };
    function Plugin( element, options ) {
        this.element = this.container = $(element);
        this.options = $.extend( {}, defaults, options );
        if (this.options.cookies) this.basket_name = (this.options.cookies !== true)?('anura-basket-'+this.options.cookies):'anura-basket';
        if (this.options.local_storage) this.basket_name = (this.options.local_storage !== true)?('anura-basket-'+this.options.local_storage):'anura-basket';
        if (this.strings === undefined) this.strings = [];
        this.strings_locale = this.options.locale;
        this.overlay = null;
        this.basket_assets = null;
        this.dlf_cache = {};
        this._defaults = defaults;
        this._name = pluginName;
        this.init();
    }
    Plugin.prototype = {
        init: function() {
            var _this = this;
            if (!$.isEmptyObject(this.strings) && this.options.locale === this.strings_locale) {
                _this.loadBasket();
            } else {
                $.anura.getJSON(this, this.options.server + '/asset' + this.options.ext + 'callback=?&string=' + ['MENUITEMID_DOWNLOAD', 'downloadformatgroup.all.downloadFormat',
                        'downloadformatmanagement.downloadformat.name', 'search.cleanUp', 'MENUITEMID_SEND', 'MENUITEMID_MAILTO', 'common.form.inactive', 'externallinks.error.nopermission.asset',
                        'anura.download.mailinput.mail', 'anura.download.mailinput.text', 'interceptor.disclaimerconfirmation', 'MENUITEMID_SHARE', 'common.copyToClipboard',
                        'downloadFormatInterceptor.messageBox.title', 'downloadFormatInterceptor.messageBox.message', 'share.newPin', 'pinaccess.name.fallback'].join('&string='),
                    {locale: this.options.locale}, 'anuraBasketStrings', function (data) {
                        _this.strings = data;
                        _this.strings_locale = _this.options.locale;
                        _this.loadBasket(_this.element.data('assets'));
                    });
            }
        },
        loadBasket: function(reload) {
            var _this = this;
            this.options.counter = this.element.find('.anura-counter, .counter').first();
            if (!this.options.counter.length) this.options.counter = this.element.find('span').first();
            if (this.element.data('assets') == null || reload) {
                this.element.data('assets', []);
                $('#'+this.basket_name+'-overlay').remove();
                this.element.off('click').on('click', function() {_this.togglePopup()});
                this.overlay = $('<div id="'+this.basket_name+'-overlay"/>').addClass('anura-basket-overlay').appendTo($('body'))
                    .on('click', function(e) {if (e.target.id === _this.basket_name+'-overlay') _this.togglePopup();});
                var popup = $('<div />').addClass('anura-basket-popup anura-locale-'+this.options.locale).appendTo(this.overlay);
                var titlebar = $('<div />').addClass('anura-basket-popup-titlebar').appendTo(popup);
                $('<span />').addClass('anura-basket-popup-title').text((_this.strings['MENUITEMID_DOWNLOAD'] || 'Download')).appendTo(titlebar);
                $('<span />').addClass('anura-basket-popup-close').appendTo(titlebar).on('click', function() {_this.togglePopup();});
                var dlfData = _this.getDlfDropdown(titlebar);
                $('<div />').addClass('anura-basket-dlf-select-option').text(this.strings['downloadformatgroup.all.downloadFormat']).appendTo(dlfData.active);

                var main = $('<div>').addClass('anura-basket-main').appendTo(popup);
                this.basket_assets = $('<div />').addClass('anura-basket-assets').appendTo(main);
                this.addTerms($('<div />').addClass('anura-basket-toc').appendTo(main));

                var footer = $('<div />').addClass('anura-basket-footer').appendTo(popup);
                $('<span />').text((_this.strings['search.cleanUp'] || 'Clear'))
                    .addClass('anura-basket-button anura-basket-reset').appendTo(footer)
                    .on('click', function() {_this.clear()});
                if (this.options.require_email) {
                    popup.addClass('anura-basket-footer-two-lines');
                    $('<input type="email" required/>').addClass('anura-basket-input').appendTo(footer)
                        .attr('placeholder', (_this.strings['anura.download.mailinput.mail'] || 'E-Mail Address'))
                        .attr('title', (_this.strings['anura.download.mailinput.text'] || ''))
                        .appendTo($('<div/>').addClass('anura-basket-mail-input').appendTo(footer));
                }
                $('<span />').text((_this.strings['MENUITEMID_DOWNLOAD'] || 'Download'))
                    .addClass('anura-basket-button anura-basket-download').appendTo(footer)
                    .on('click', function() {
                        if (!_this.checkRequiredFields()) return;
                        var link = _this.getDownloadUrl(_this.getDownloads(_this.element.data('assets')), null);
                        if (!link) return; // nothing to do
                        var opened = window.open(link, 'blank_'+new Date().getTime());
                        if (opened) {
                            $.each(_this.getDownloads(_this.element.data('assets')), function(i, d) {
                                $(window).trigger('anura-download', [d.asset, d.dlf, d.version]);
                            });
                            if (_this.options.clear_after_download) _this.clear();
                        }
                    });
                if (_this.options.send) {
                    $('<span />').addClass('anura-basket-button anura-basket-mailto').text(_this.strings['MENUITEMID_SEND'] || 'Send')
                        .attr('title', _this.strings['MENUITEMID_MAILTO']).on('click', function() {
                        if (!_this.checkRequiredFields()) return;
                        var link = (_this.options.send === 'share') ? _this.updateShareUrl(true) : _this.getDownloadUrl(_this.getDownloads(_this.element.data('assets')), null);
                        if (!link) return; // nothing to do
                        window.location.href = 'mailto:?body='+encodeURIComponent(link);
                    }).appendTo(footer);
                }
                if (_this.options.pin) {
                    var pinbutton = $('<span />').addClass('anura-basket-button anura-basket-pin').text(_this.strings['pinaccess.name.fallback'] || 'PIN-Link')
                        .attr('title', _this.strings['share.newPin']).on('click', function() {
                            if (!_this.checkRequiredFields()) return;
                            var link = _this.options.server+'/pin'+_this.options.ext+'&create='+_this.element.data('assets').join(',');
                            if ($.anura.tokenProvider) link += '&token='+$.anura.tokenProvider();
                            if (_this.options.pin === 'send') {
                                if (pinbutton.hasClass('anura-loading')) return;
                                pinbutton.addClass('anura-loading');
                                $.anura.getJSON(_this, link+'&callback=?',{}, 'anuraBasketPin', function (data) {
                                    window.location.href = 'mailto:?subject=PIN%20'+data.pin+'&body='+encodeURIComponent(data.link);
                                    if (_this.options.clear_after_download) _this.clear();
                                    pinbutton.removeClass('anura-loading');
                                }, function () {
                                    pinbutton.removeClass('anura-loading');
                                });
                            } else {
                                var opened = window.open(link+'&redirect=true', 'blank_'+new Date().getTime());
                                if (opened && _this.options.clear_after_download) _this.clear();
                            }
                        }).appendTo(footer);
                }
                if (_this.options.share) {
                    if (!ClipboardJS.isSupported()) return;
                    var link = $('<span />').addClass('anura-basket-button anura-basket-share').text(_this.strings['MENUITEMID_SHARE'] || 'Share')
                        .attr('title', _this.strings['common.copyToClipboard']).attr('data-clipboard-target', '#anura-clipboard-target-basket').appendTo(footer);
                    $('<input class="anura-clipboard-target" id="anura-clipboard-target-basket"/>').prop('readonly', true).appendTo(link);
                    var setLinkState = function (e, link, clazz) {
                        e.clearSelection(); _this.updateShareUrl();
                        link.removeClass('anura-success anura-error').addClass(clazz);
                        setTimeout(function () {link.removeClass('anura-success anura-error')}, 2500);
                    };
                    new ClipboardJS('.anura-basket-button.anura-basket-share')
                        .on('success', function(e) {setLinkState(e, link, 'anura-success');})
                        .on('error', function(e) {setLinkState(e, link, 'anura-error');});
                }
                if ($.anura.hasState('', 'basket')) {
                    var addAllCapable = [];
                    $.each($.anura.getState('', 'basket').split(','), function (index, item) {
                        var dl_asset = item.split(':'); // special case with asset version specified
                        if (dl_asset.length > 1) _this.add(dl_asset[0], dl_asset[1]);
                        else addAllCapable.push(dl_asset);
                    });
                    if (addAllCapable.length) _this.addAll(addAllCapable);
                    _this.togglePopup();
                    $.anura.removeState('', 'basket');
                } else {
                    if (_this.options.cookies && $.cookie && $.cookie(this.basket_name)) {
                        _this.options.add = JSON.parse($.cookie(this.basket_name));
                    }
                    if (_this.options.local_storage && typeof localStorage !== 'undefined' && localStorage.getItem(this.basket_name)) {
                        _this.options.add = JSON.parse(localStorage.getItem(this.basket_name));
                    }
                    if (reload) _this.options.add = reload;
                }
            }
            if (this.options.add) {
                if ($.isArray(this.options.add)) this.addAll(this.options.add);
                else this.add(this.options.add);
                this.options.add = null;
            }
            if (this.options.remove) {
                if ($.isArray(this.options.remove)) {
                    $.each(this.options.remove, function() {_this.remove(this)});
                } else {
                    this.remove(this.options.remove);
                }
                this.options.remove = null;
            }
            if (this.options.action) {
                switch (this.options.action) {
                    case 'show': this.overlay.show(); break;
                    case 'hide': this.overlay.hide(); break;
                    case 'toggle': this.togglePopup(); break;
                    case 'clear': this.clear(); break;
                }
                this.options.action = null;
            }
            _this.element.trigger('anura-loaded', [_this.options]);
        },
        getDownloadUrl: function(downloads, url) {
            if (downloads.length === 0) return null;
            var allSameDlf = true; var prevDlf = null;
            downloads.map(function (d) {
              allSameDlf = prevDlf === null || prevDlf === d.dlf || d.crop || d.version;
              prevDlf = d.dlf;
            })
            return this.options.server+'/'+(url?url:('asset'+this.options.ext))+
                'download='+downloads.map(function(d) {
                    var suboptions = [];
                    if (d.version) suboptions.push('version='+d.version);
                    if (d.crop) suboptions.push('crop='+d.crop);
                    return d.asset+(allSameDlf ? '' : ':'+d.dlf+(suboptions.length?':'+suboptions.join(';'):''));
                }).sort().join(',')
                +(allSameDlf ? '&format='+prevDlf : '')
                +(this.options.force_zip?'&forcezip=true':'')
                +($.anura.tokenProvider?'&token='+$.anura.tokenProvider():'')
                +(this.options.require_email?('&email='+encodeURIComponent(this.overlay.find('.anura-basket-mail-input input').val())):'')
                +(this.options.locale?('&locale='+this.options.locale):'');
        },
        getDownloads: function(assets) {
            var downloads = [], _this = this;
            $.each(assets, function(index, asset) {
                var selected = $('#'+_this.basket_name+'-asset-'+asset+' .anura-basket-dlf-select-active .anura-basket-dlf-select-option');
                var dlf = selected.data('id');
                var version = $('#'+_this.basket_name+'-asset-'+asset+' .anura-basket-version-select-active .anura-basket-version-select-option').data('id');
                if (dlf) downloads.push({asset: asset, dlf: dlf, version: version, crop: selected.data('crop')});
            });
            return downloads;
        },
        add: function(asset, preset_dlf = this.options.default_format, version = null, update_share_url = true) {
            var assets = this.element.data('assets'), _this = this;
            if (typeof asset === "string" && asset.indexOf(":") !== -1) {
                var assetOpts = asset.split(":"); // assetId:downloadFormatId:version
                asset = assetOpts[0];
                if (assetOpts[1]) preset_dlf = assetOpts[1];
                if (assetOpts[2]) version = assetOpts[2];
            }
            if (!asset || $.inArray(asset, assets) !== -1) return; // prevent duplicates
            if (this.options.max_assets > 0 && assets.length >= this.options.max_assets) {
                var remove = assets.slice(0, assets.length - this.options.max_assets + 1);
                $.each(remove, function(index, item) {_this.remove(item);});
            }
            assets.push(asset);
            $('#basket'+asset).addClass('added');
            this.setCount(assets.length);
            if (this.options.cookies && $.cookie) $.cookie(this.basket_name, JSON.stringify(assets), {path: '/'});
            if (this.options.local_storage && typeof localStorage !== 'undefined') localStorage.setItem(this.basket_name, JSON.stringify(assets));
            var addAssetFunc = function (dlfFor) {
                if (!dlfFor) return;
                if (!_this.dlf_cache[asset]) _this.dlf_cache[asset] = dlfFor;
                var container = _this.addAssetContainer(asset, dlfFor.name);
                container.addClass('anura-asset-type-'+dlfFor.type).data('asset-extension', dlfFor.extension);
                var dlfData = _this.getDlfDropdown(container);
                if (!dlfFor.formats || dlfFor.formats.length === 0) {
                    $('<div/>').addClass('anura-basket-dlf-select-option').text(_this.strings['common.form.inactive']).appendTo(dlfData.active);
                    dlfData.wrapper.addClass('anura-basket-dlf-unavailable');
                } else {
                    $.each(dlfFor.formats, function(index, item) {
                        var is_editable = item.features && $.inArray('editable', item.features) !== -1;
                        var opt = $('<div />').attr('data-id', item.id)
                            .css('background-image', 'url("'+_this.options.server+'/'+_this.options.icon_transform(item.icon)+'")')
                            .addClass('anura-basket-dlf-select-option').appendTo(dlfData.options)
                            .on('click', function() {
                                var active = $(this).clone().appendTo(dlfData.active.empty());
                                if (is_editable && jQuery().anuraCropper) {
                                    _this.element.anuraCropper({
                                        server: _this.options.server,
                                        locale: _this.options.locale,
                                        assetId: asset,
                                        downloadFormat: item.id,
                                        mode: 'apply',
                                        callback: function (options, geo, param) {
                                            active.data('crop', param);
                                        }
                                    })
                                }
                            });
                        if (item.description && _this.options.show_description) {
                            opt.addClass('anura-dlf-with-descr');
                            $('<span/>').addClass('anura-dlf-name').text(item.name).attr('title', item.name).appendTo(opt);
                            $('<span/>').addClass('anura-dlf-descr').text(item.description).attr('title', item.description).appendTo(opt);
                        } else {
                            opt.text(item.name).attr('title', item.description);
                        }
                        if ((!preset_dlf && index === 0) || preset_dlf == /*not ===*/ item.id) {// select active entry
                            opt.clone().appendTo(dlfData.active);
                        }
                        if (!_this.overlay.find('.anura-basket-popup-titlebar .anura-basket-dlf-select-option[data-id='+item.id+']').length && !is_editable) { // add to "all dlfs"
                            opt.clone().appendTo(_this.overlay.find('.anura-basket-popup-titlebar .anura-basket-dlf-select-options'))
                                .on('click', function() {
                                    _this.overlay.find('.anura-basket-asset .anura-basket-dlf-select-option[data-id='+item.id+']').click();
                                });
                        }
                    });
                }
                if (dlfFor.infofields) $.each(dlfFor.infofields, function(infoId, infoVal) {container.data('asset-infofield-'+infoId, infoVal);});
                if (dlfFor.infofields_raw) $.each(dlfFor.infofields_raw, function(infoId, infoVal) {container.data('asset-infofield-raw-'+infoId, infoVal);});
                _this.element.trigger('anura-asset-added', [_this.options, asset, dlfFor, version]);
                if (update_share_url) _this.updateShareUrl();
            }
            var removeAssetFunc = function(fail) {
                _this.remove(asset);
                if (_this.options.show_failed) {
                    var fail_msg = _this.strings['externallinks.error.nopermission.asset'] || 'You have no permission for asset with id: {0}!';
                    _this.addAssetContainer(asset, fail_msg.replace(/\{0\}\!?/g, asset)).addClass('anura-asset-failed');
                }
                _this.element.trigger('anura-asset-failed', [_this.options, asset]);
                if (update_share_url) _this.updateShareUrl();
            }
            if (this.dlf_cache[asset]) {
                addAssetFunc(this.dlf_cache[asset])
            } else {
                $.anura.getJSON( this, this.options.server+'/asset'+this.options.ext+'callback=?',
                    {downloadformats_for: asset, locale: this.options.locale, alt_name: this.options.alternative_name.id,
                        infofields: this.options.infofields?this.options.infofields.join(','):''},
                    'anuraBasketPopupDlfFor'+asset, addAssetFunc, removeAssetFunc);
            }
        },
        addAll(assets) {
            var _this = this;
            if (!$.isArray(assets) || assets.length === 1) {
                this.add(assets);
            } else if (!this.version || this.version < 2.8) {
                $.each(this.options.add, function() {_this.add(this)});
            } else {
                var batchSize = 200; var batchArrays = [];
                for (var i = 0; i < assets.length; i += batchSize) {
                    batchArrays.push(assets.slice(i, i + batchSize));
                }
                $.each(batchArrays, function (index, batch) {
                    $.anura.getJSON( _this, _this.options.server+'/asset'+_this.options.ext+'callback=?',
                        {downloadformats_for: batch.join(','), locale: _this.options.locale, alt_name: _this.options.alternative_name.id,
                            infofields: _this.options.infofields?_this.options.infofields.join(','):''},
                        'anuraBasketPopupDlfFor'+index+Date.now(), function(dlfsFor) {
                            $.each(batch, function (index, asset) {
                                _this.dlf_cache[asset] = dlfsFor[asset];
                                _this.add(asset, _this.options.default_format, null, false);
                            });
                        }, function () {
                            console.error("Failed to fetch download formats for "+assets)
                        });
                    _this.updateShareUrl();
                });
            }
        },
        addAssetContainer: function(id, name) {
            var _this = this;
            var container = $('<div />').attr('id', this.basket_name+'-asset-'+id)
                .addClass('anura-basket-asset').appendTo(this.basket_assets);
            var tWrap = $('<div/>').addClass('anura-basket-thumb-wrap').appendTo(container);
            $('<img />').text(name).attr('title', name).attr('loading', 'lazy')
                .attr('src', this.options.server+'/asset'+this.options.ext+'thumb='+id+($.anura.tokenProvider?'&token='+$.anura.tokenProvider():'')+'&img=a.jpg')
            /* IE */.removeAttr('width').removeAttr('height')/* /IE */
                .appendTo($('<div/>').addClass('anura-basket-thumb').appendTo(tWrap));
            $('<span />').addClass('anura-basket-asset-remove').html(this.options.close_icon)
                .on('click', function(e) {e.stopPropagation(); _this.remove(id);}).appendTo(container);
            $('<p />').addClass('anura-basket-asset-name').text(name).attr('title', name).appendTo(container);
            return container;
        },
        getDlfDropdown: function(wrapperTarget) {
            var _this = this;
            var dlfWrap = $('<div />').addClass('anura-basket-dlf-select').appendTo(wrapperTarget)
                .attr('title', this.strings['downloadformatmanagement.downloadformat.name'])
                .on('click', function(e) {
                    var state = $(this).hasClass('anura-select-active');
                    _this.overlay.find('.anura-select-active').removeClass('anura-select-active');
                    $(this).toggleClass('anura-select-active', !state && $(e.target).parents('.anura-basket-dlf-select-options').length === 0);
                });
            var dlfActive = $('<div />').addClass('anura-basket-dlf-select-active').appendTo(dlfWrap);
            var dlfOpts = $('<div />').addClass('anura-basket-dlf-select-options').appendTo(dlfWrap);
            return {wrapper: dlfWrap, active: dlfActive, options: dlfOpts};
        },
        remove: function(asset) {
            if (!asset) return;
            var assets = this.element.data('assets');
            var array_pos = $.inArray(asset, assets);
            if (array_pos !== -1) assets.splice(array_pos, 1);
            $('#'+this.basket_name+'-asset-'+asset).remove();
            this.setCount(assets.length);
            this.updateShareUrl();
            $('#basket'+asset).removeClass('added');
            if (assets.length === 0 && this.overlay.is(':visible')) this.togglePopup();
            if (this.options.cookies && $.cookie) {
                if (assets.length > 0) $.cookie(this.basket_name, JSON.stringify(assets), {path: '/'});
                else $.removeCookie(this.basket_name, {path: '/'});
            }
            if (this.options.local_storage && typeof localStorage !== 'undefined') {
                if (assets.length > 0) localStorage.setItem(this.basket_name, JSON.stringify(assets));
                else localStorage.removeItem(this.basket_name);
            }
            this.element.trigger('anura-asset-removed', [this.options, asset]);
        },
        clear: function() {
            this.element.data('assets', []);
            if (this.options.cookies && $.removeCookie) $.removeCookie(this.basket_name, {path: '/'});
            if (this.options.local_storage && typeof localStorage !== 'undefined') localStorage.removeItem(this.basket_name);
            this.overlay.find('.anura-basket-asset').remove();
            this.setCount(0);
            this.updateShareUrl();
            $('.anura-add-basket.added').removeClass('added'); // gallery indicators
            this.togglePopup();
            this.element.trigger('anura-cleared', [this.options]);
        },
        setCount: function(count) {
            if (this.options.counter) this.options.counter.html(count);
            this.element.removeClass('anura-basket-empty anura-basket-full')
                .addClass(count === 0 ? 'anura-basket-empty' : 'anura-basket-full');
        },
        updateShareUrl: function(force_update) {
            if (this.options.share || force_update) {
                var clipboard_target = $('#anura-clipboard-target-basket'), url;
                var downloadQuery = this.getDownloads(this.element.data('assets')).map(function (d) {
                    return d.asset + ':' + d.dlf + (d.version ? ':'+d.version : '');
                }).sort().join(',');
                if (downloadQuery) {
                    $.anura.setState('', 'basket', downloadQuery);
                    url = window.location.href;
                    clipboard_target.val(url);
                    $.anura.removeState('', 'basket');
                    return url;
                } else {
                    clipboard_target.val('');
                }
            }
        },
        togglePopup: function() {
            if (this.overlay.is(':visible') || !this.element.data('assets').length) {
                this.overlay.hide();
                $('#'+this.basket_name+'-tc-confirm-cb').prop('checked', false);
                this.basket_assets.show();
                this.overlay.find('.anura-basket-toc').hide();
            } else {
                this.overlay.show();
            }
            this.updateShareUrl();
            this.element.trigger('anura-toggled');
        },
        addTerms: function(target) {
            var tc = this.options.terms_and_conditions, loc = this.options.locale, _this = this;
            if (!tc) return;
            if (tc === true && _this.strings['downloadFormatInterceptor.messageBox.message']) tc = {
                title: _this.strings['downloadFormatInterceptor.messageBox.title'],
                message: _this.strings['downloadFormatInterceptor.messageBox.message'],
                confirm: _this.strings['interceptor.disclaimerconfirmation']
            };
            if (tc.title) $('<h3 />').addClass('anura-basket-tc-title').html((tc.title[loc] ? tc.title[loc] : tc.title)+':').appendTo(target);
            $('<p />').addClass('anura-basket-tc-msg').html(tc.message[loc] ? tc.message[loc] : tc.message).appendTo(target);
            if (tc.confirm) {
                var confirm = $('<p/>').addClass('anura-basket-tc-confirm').appendTo(target);
                $('<input type="checkbox" required/>').attr('id', this.basket_name+'-tc-confirm-cb').appendTo(confirm).on('change', function() {
                    _this.overlay.find('.anura-basket-tc-wrapper').toggleClass('anura-tc-confirmed');
                });
                $('<label />').attr('for', this.basket_name+'-tc-confirm-cb').html(' '+(tc.confirm[loc] ? tc.confirm[loc] : tc.confirm)).appendTo(confirm);
            }
        },
        checkRequiredFields: function() {
            if (this.options.require_email) {
                var mail = this.overlay.find('.anura-basket-mail-input input');
                if (!mail.val()) {
                    this.shake(mail);
                    return false;
                }
            }
            if (this.options.terms_and_conditions && (!$.isFunction(this.options.terms_and_conditions.show) || this.options.terms_and_conditions.show())) {
                var cb = $('#'+this.basket_name+'-tc-confirm-cb');
                var cb_checked = cb.length && cb.is(':checked');
                if (this.basket_assets.is(':visible') && !cb_checked) {
                    this.basket_assets.slideUp();
                    this.overlay.find('.anura-basket-toc').slideDown();
                    return false;
                }
                if (!cb_checked) {
                    this.shake(cb);
                    return false;
                } else if ($.isFunction(this.options.terms_and_conditions.confirmed)) {
                    this.options.terms_and_conditions.confirmed();
                }
            }
            return true;
        },
        shake: function(item) {
            var _item = item; // yay scoping
            if (item.attr('type') !== 'checkbox') item.css('border-color', 'red');
            item.addClass('anura-animated anura-shake')
                .one('webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend', function () {
                    _item.removeClass('anura-animated anura-shake').css('border-color', '');
                });
        }
    };
    $.fn[pluginName] = function ( options ) {
        return this.each(function (i, item) { // iterate matches
            var instance = $.data(item, "plugin_" + pluginName);
            if (!instance) { // if it has no instance yet, create it
                $.data(item, "plugin_" + pluginName, new Plugin( item, options ));
            } else { // otherwise merge new options with existing ones, empty and re-init
                instance.options = $.extend( {}, instance.options, options );
                instance.init();
            }
        });
    };
})( jQuery, window, document );
