/* © brix cross media AG, http://www.brix.ch - It is strictly forbidden to change, reuse or copy the source code of this document. */
;(function ( $, window, document, undefined ) {
    var pluginName = "anuraBreadcrumbs",
        defaults = {
            server: 'https://celum.local/anura/first', // which anura endpoint to use
            tree: null, // the anuraTree to listen on (may also be a container containing multiple trees)
            multiMode: 'local', // when to switch to multi-mode (tags instead of hierarchy), one of: "local" (per tree), "global" (over all trees) or "always"
            callback: null // what happens when a node/leaf is clicked function(server, kind, id, name) or NULL to click the tree
        };
    function Plugin( element, options ) {
        this.element = $(element);
        this.options = $.extend( {}, defaults, options );
        this._defaults = defaults;
        this._name = pluginName;
        this.init();
    }
    Plugin.prototype = {
        init: function() {
            var _this = this;
            this.element.empty();
            this.container = $('<div/>').addClass('anura-breadcrumbs-container').appendTo(this.element);
            var onUpdate = function() {
                _this.container.empty();
                if (_this.options.multiMode === 'local') {
                    _this.options.tree.find('.anura-tree-container').each(function () {
                        _this.toCrumbs($(this));
                    });
                } else {
                    _this.toCrumbs(_this.options.tree);
                }
            };
            this.options.tree.on('anura-loaded anura-reset', function() {
                _this.options.server = _this.options.tree.data('server');
                _this.options.kind = _this.options.tree.data('kind');
                _this.container.empty();
            }).on('anura-init anura-clicked anura-keywords-clicked anura-facet-update anura-loaded', onUpdate);
            onUpdate();
        },
        toCrumbs: function(treeSource) {
            var _this = this;
            var container = $('<ul />').addClass('anura-breadcrumbs').appendTo(_this.container);
            var currents = treeSource.find('li.anura-current');
            if (currents.length > 1 || _this.options.multiMode === 'always') { // multi-mode
                currents.each(function() {
                    var li = $('<li />').addClass('anura-breadcrumb anura-breadcrumb-multi').appendTo(container);
                    _this.appendLink(li, $(this));
                });
            } else { // regular hierarchy
                var depth = 0;
                treeSource.find('li.anura-active').each(function() {
                    var li = $('<li />').addClass('anura-breadcrumb anura-breadcrumb-levels anura-breadcrumb-level-'+(depth++)).appendTo(container);
                    _this.appendLink(li, $(this))
                });
            }
        },
        appendLink: function (li, sourceNode) {
            var _this = this, nodeId = sourceNode.find('> a').data('id'), nodeName = sourceNode.find('> a .anura-node-name').text();
            var a = $('<a />').text(nodeName).data('id', nodeId).appendTo(li);
            var facetCount = sourceNode.find('> a .anura-asset-count'); // should we remove on 0?
            if (facetCount) a.html(nodeName + ' <span class="anura-asset-count">'+facetCount.text()+'</span>');
            a.on('click', function() {
                if (_this.options.callback) {
                    _this.options.callback(_this.options.server, _this.options.kind, nodeId, nodeName);
                } else if ($(this).parent().hasClass('anura-breadcrumb-multi')) {
                    _this.options.tree.find('#anura-node-'+nodeId+' > a').trigger($.Event("click", { ctrlKey: true }));
                } else {
                    _this.options.tree.find('#anura-node-'+nodeId+' > a').click();
                }
                _this.element.trigger('anura-clicked', [_this.options.server, _this.options.kind, nodeId, nodeName]);
            });
            var pathName = nodeName;
            sourceNode.parentsUntil('.anura-tree-container', 'li').each(function (index, parent) {
                pathName = $(parent).find('> a .anura-node-name').text() + ' > ' + pathName;
            });
            a.attr('title', pathName);
        }
    };
    $.fn[pluginName] = function ( options ) {
        return this.each(function (i, item) { // iterate matches
            var instance = $.data(item, "plugin_" + pluginName);
            if (!instance) { // if it has no instance yet, create it
                $.data(item, "plugin_" + pluginName, new Plugin( item, options ));
            } else { // otherwise merge new options with existing ones, empty and re-init
                instance.options = $.extend( {}, instance.options, options );
                instance.init();
            }
        });
    };
})( jQuery, window, document );
