/* © brix cross media AG, http://www.brix.ch - It is strictly forbidden to change, reuse or copy the source code of this document. */
;(function ( $, window, document, undefined ) {
    var pluginName = "anuraDetails",
        defaults = {
            server: 'https://celum.local/anura/first', // REQUIRED, which anura endpoint to use
            ext: '.do?', // extension of server pages (.php, .do)
            asset: 0, version: null,
            locale: '', // ISO 639-1 code, blank for default
            show: ['general','infofields','downloads'], // which detail components to show and in what order
                // any of: infofields, general, keywords, keyword_paths, keyword_path_links, pages, share, downloads, links, similar, metadata, get_reviews, post_reviews, function (asset_id, metadata, table, options) {}
            metadata: [], // for "show: metadata", configure which fields should be shown in what order, e.g. ['name', 'created', 'info_123']
            infofield_transform: {}, // function to transform infofield values per ID, e.g. {3: function(info) {return 'Hello '+info.value+'!'}}
            //infofield_sort: function(a, b) {return a.name.localeCompare(b.name, undefined, {numeric: true, sensitivity: 'base'})}, // custom infofield sorting
            basket: null, // object to use as a basket, initialized by a basket plugin
            send: false, // add a mailto-link (ID of the download format to use or true for the first)
            split: false, // put the widgets into individual tables instead of one big one
            alternative_name: 0, // id of the alternative name infofield (only for show:links)
            download_directly: true, // make the different formats available directly
            download_relations: false, // make relation package download available
            force_zip: false, // force even single asset downloads to be delivered in a ZIP-file
            icon_transform: function(icon) {return icon.replace('/big/', '/small/')},
            show_dlf_description: true, // show the description of a download format as a separate line (instead of a tooltip)
            scrollfix: true, // on some layouts the page jumps when you paginate
            paginator_nav: {prev: '&larr;', next: '&rarr;'},
            paginator_target: '#cboxContent .cboxPhoto, .anura-lightbox-embed .anura-active',
            keywordPathLinkSep: '&hairsp;/&hairsp;',
            callback: false, // when details are shown
            state: '', // prefix for state in url (multi-instance) or false to disable state
            caller: null, // set by anuraLightbox when shown in its sidebar
            downloadFormat: 1, // download format used by version downloads
            noderef_callback: function() {
                $('.anura-tree-container #anura-node-'+$(this).data('id')+':not(.anura-current) > a, #cboxClose, #baguetteBox-overlay #close-button, .anura-lightbox-button.anura-lightbox-close').click(); // update any trees, close details
                $('.anura-search-noderef option[value="'+$(this).data('id')+'"]').prop('selected', true).trigger('change').parent().trigger('chosen:updated'); // update search dropdowns and chosen
            },
            relations: {
                type: null,
                view: 'anuraGallery',
                view_settings: {}
            }
        };
    function Plugin( element, options ) {
        this.element = this.container = $(element);
        this.options = $.extend( {}, defaults, options );
        this.assets = [];
        if (this.strings === undefined) this.strings = [];
        this.strings_locale = this.options.locale;
        this._defaults = defaults;
        this._name = pluginName;
        this.init();
    }
    Plugin.prototype = {
        init: function() {
            var _this = this;
            this.element.empty();
            this.container = $('<div />').addClass('anura-details-container anura-loading').appendTo(this.element)
                .removeClass(function (index, css) {return (css.match (/anura-locale-\S+/g) || []).join(' ');}).addClass('anura-locale-'+this.options.locale);
            if (!$.isEmptyObject(this.strings) && this.options.locale === this.strings_locale) {
                this.requestDetails(this.options.asset);
            } else {
                $.anura.getJSON(this, this.options.server + '/asset' + _this.options.ext + 'callback=?&string=anura.addToBasket&string=anura.removeFromBasket' +
                    '&string=MENUITEMID_SHARE&string=MENUITEMID_METADATA&string=MENUITEMID_MAILTO&string=search.selectAll&string=PdfDocumentProperties.pages' +
                    '&string=paging.page&string=anura.similarAssets&string=common.copyToClipboard&string=detailview.assetversion.title' +
                    '&string=detailview.assetversion.version&string=detailview.assetversion.date&string=detailview.assetversion.current' +
                    '&string=assetReview.view.asset.label.ratingAverage&string=assetReview.view.asset.title' +
                    '&string=assetReview.view.asset.label.comment&string=assetReview.view.asset.label.rating' +
                    '&string=common.actions.clear&string=common.actions.save&string=assetReview.view.asset.label.checkReviewError' +
                    '&string=assetReview.view.asset.headerNewReview&string=assetReview.view.asset.headerReviewList&string=anura.relations',
                    {locale: this.options.locale}, 'anuraDetailsStrings', function (data) {
                        _this.strings = data;
                        _this.strings_locale = _this.options.locale;
                        _this.requestDetails(_this.options.asset);
                    });
            }
        },
        requestDetails: function(asset_id) {
            var _this = this;
            $.anura.getJSON(this, this.options.server+'/asset'+this.options.ext+'callback=?', {
                    details: asset_id,
                    locale: this.options.locale,
                    alt_name: this.options.alternative_name,
                    details_infofields: this.options.show.infofields || $.inArray('infofields', this.options.show) !== -1 || $.inArray('metadata', this.options.show) !== -1,
                    details_keywords: this.options.show.keywords || $.inArray('keywords', this.options.show) !== -1,
                    details_keyword_paths: $.inArray('keyword_paths', this.options.show) !== -1,
                    details_keyword_list: $.inArray('keyword_links', this.options.show) !== -1 || $.inArray('keyword_path_links', this.options.show) !== -1 ||  $.inArray('similar', this.options.show) !== -1,
                    details_links: this.options.show.links || $.inArray('links', this.options.show) !== -1,
                    details_versions: $.inArray('versions', this.options.show) !== -1,
                    // details_reviews: $.inArray('reviews', this.options.show) !== -1
                }, 'anuraDetails', function(data) {
                    if (!data) return;
                    _this.container.find('table').remove();
                    $.anura.pushState(_this.options.state, 'details', asset_id);
                    _this.table = $('<table />').addClass('anura-table-0').appendTo(_this.container);
                    if ($.isArray(_this.options.show)) { // new syntax
                        $.each(_this.options.show, function(index, value) {
                            if (_this.options.split) _this.table.addClass('anura-table-'+value);
                            switch (value) {
                                case 'general': _this.addGenerals(asset_id, data); break;
                                case 'infofields': _this.addInfofields(asset_id, data); break;
                                case 'keyword_paths': _this.addKeywordPaths(asset_id, data); break;
                                case 'keyword_links': _this.addKeywordLinks(asset_id, data); break;
                                case 'keyword_path_links': _this.addKeywordPathLinks(asset_id, data); break;
                                case 'keywords': _this.addKeywords(asset_id, data); break;
                                case 'pages': _this.addPages(asset_id, data); break;
                                case 'share': _this.addShare(asset_id, data); break;
                                case 'downloads': _this.addDownloads(asset_id, data); break;
                                case 'links': _this.addLinks(asset_id, data); break;
                                case 'relations': _this.addRelations(asset_id, data); break;
                                case 'similar': _this.addSimilar(asset_id, data); break;
                                case 'metadata': _this.addMetadata(asset_id, data); break;
                                case 'versions': _this.addVersions(asset_id, data); break;
                                case 'get_reviews': _this.addGetReviews(); break;
                                case 'post_reviews': _this.addPostReviews(); break;
                                default:
                                    if ($.isFunction(value)) value(asset_id, data, _this.table, _this.options);
                                    else if (console) console.error("unknown detail key "+value);
                            }
                            if (_this.options.split && index < _this.options.show.length - 1) {
                                _this.table = $('<table />').addClass('anura-table-'+(index+1)).appendTo(_this.container);
                            }
                        });
                    } else { // old syntax
                        if (_this.options.show.general) _this.addGenerals(asset_id, data);
                        if (_this.options.show.infofields) _this.addInfofields(asset_id, data);
                        if (_this.options.show.keyword_paths) _this.addKeywordPaths(asset_id, data);
                        else if (_this.options.show.keywords) _this.addKeywords(asset_id, data);
                        if (_this.options.show.downloads) _this.addDownloads(asset_id, data);
                    }
                    _this.container.removeClass('anura-loading').data('id', asset_id);
                    _this.element.trigger('anura-loaded', [_this.options, data]);
                    if (_this.options.callback) {_this.options.callback(_this.options, data);}
                });
        },
        addGenerals: function(asset, data) {
            var _this = this;
            $('<td />').attr('colspan', 2).html(data.general_title).appendTo($('<tr />').addClass('title general').appendTo(this.table));
            $.each(data.general, function(index, general) {_this.addGeneral(general)});
        },
        addGeneral: function(general) {
            if (!general) return;
            if (general.id) {
                if (general.id === 'downloadable' || general.id === 'custom' || (general.id === 'pages' && $.inArray('pages', this.options.show) >= 0 && general.value > 1)) return;
                this.addRow(general.name, general.value, 'general '+general.id);
            } else { // legacy
                this.addRow(index, general, 'general');
            }
        },
        addInfofields: function(asset, data) {
            if ($.isEmptyObject(data.infofields)) return;
            var _this = this;
            $('<td />').attr('colspan', 2).html(data.infofields_title).appendTo($('<tr />').addClass('title infofields').appendTo(this.table));
            if ($.isFunction(this.options.infofield_sort)) data.infofields.sort(this.options.infofield_sort);
            $.each(data.infofields, function(index, infofield) {_this.addInfofield(infofield)});
        },
        addInfofield: function(infofield) {
            if (!infofield) return;
            if (infofield.id) {
                if ($.isFunction(this.options.infofield_transform[infofield.id.replace('info_', '')])) {
                    this.options.infofield_transform[infofield.id.replace('info_', '')](infofield);
                }
                if (infofield.values) { // node referencing
                    var nodeRefLinks = $('<ul />').addClass('noderef');
                    var _this = this;
                    $.each(infofield.values, function(ind, node) {
                        var isClickable = _this.isNodeClickable(node.value);
                        var nodeRef = $(isClickable?'<a/>':'<span/>').html(node.name).attr('title', node.path)
                            .data('id', node.value).data('root', node.root).data('type', node.type).attr('id', node.id)
                            .appendTo($('<li />').appendTo(nodeRefLinks));
                        if (isClickable) nodeRef.on('click', _this.options.noderef_callback);
                    });
                    this.addRow(infofield.name, nodeRefLinks, 'infofield '+infofield.id, infofield.info);
                } else if (infofield.value) {
                    this.addRow(infofield.name, infofield.value.replace(/>\\n</, '><').replace(/\\n/, '<br>'), 'infofield '+infofield.id, infofield.info);
                }
            } else { // legacy
                this.addRow(index, infofield, 'infofield');
            }
        },
        addMetadata: function(asset, data) {
            var _this = this;
            $('<td />').attr('colspan', 2).html(this.strings['MENUITEMID_METADATA'] || 'Metadata')
                .appendTo($('<tr />').addClass('title metadata').appendTo(this.table));
            var lookup = {};
            $.each(data.infofields, function(index, infofield) {lookup[infofield.id] = infofield});
            $.each(data.general, function(index, general) {lookup[general.id] = general});
            $.each(this.options.metadata, function(index, metadata) {
                if (metadata.indexOf('info_') === 0) {
                    _this.addInfofield(lookup[metadata]);
                } else if (metadata === 'id' || metadata === 'asset_id') {
                    _this.addRow('ID', asset, 'general asset_id');
                } else {
                    _this.addGeneral(lookup[metadata]);
                }
            });
        },
        addKeywords: function(asset, data) {
            if ($.isEmptyObject(data.keywords)) return;
            $('<td />').attr('colspan', 2).html(data.keywords_title).appendTo($('<tr />').addClass('title keywords').appendTo(this.table));
            var keywords = '';
            $.each(data.keywords, function(name, value) {keywords += name+', '});
            $('<td />').attr('colspan', 2).html(keywords.substring(0, keywords.length-2)).addClass('keyword-list').appendTo($('<tr />').appendTo(this.table));
        },
        addKeywordPaths: function(asset, data) {
            if ($.isEmptyObject(data.keyword_paths)) return;
            var _this = this;
            $('<td />').attr('colspan', 2).html(data.keywords_title).appendTo($('<tr />').addClass('title keywords').appendTo(this.table));
            $.each(data.keyword_paths, function(index, value) {
                $('<td />').attr('colspan', 2).html(value).appendTo($('<tr />').addClass('keyword').appendTo(_this.table));
            });
        },
        addKeywordLinks: function(asset, data) {
            if ($.isEmptyObject(data.keyword_list)) return;
            var _this = this;
            $('<td />').attr('colspan', 2).html(data.keywords_title).appendTo($('<tr />').addClass('title keywords').appendTo(this.table));
            var td = $('<td />').attr('colspan', 2).addClass('keyword-links').appendTo($('<tr />').appendTo(this.table));
            $.each(data.keyword_list, function(index, value) {
                _this.asMaybeClickableLink(value).appendTo(td);
                if (index !== (data.keyword_list - 1)) td.append(", ");
            });
        },
        addKeywordPathLinks: function(asset, data) {
            if ($.isEmptyObject(data.keyword_list)) return;
            var _this = this;
            $('<td />').attr('colspan', 2).html(data.keywords_title).appendTo($('<tr />').addClass('title keywords').appendTo(this.table));
            var pathDedupe = [], termLookup = []; // deduplicate parent keywords that are assigned again further up in the path of a child keyword
            $.each(data.keyword_list, function(index, value) {pathDedupe.push(value.path); termLookup[value.name] = value;});
            $.each(data.keyword_list, function(index, value) {
                if ($.inArray(value.path+'/'+value.name, pathDedupe) === -1) {
                    var td = $('<td />').attr('colspan', 2).addClass('keyword-links keyword-path-links').appendTo($('<tr />').appendTo(_this.table));
                    $.each(value.path.split('/'), function(ix, v) {
                        if (v && v.length > 0) {
                            if (termLookup[v]) _this.asMaybeClickableLink(termLookup[v]).appendTo(td);
                            else $('<span/>').text(v).appendTo(td);
                            td.append(_this.options.keywordPathLinkSep);
                        }
                    });
                    _this.asMaybeClickableLink(value).appendTo(td);
                }
            });
        },
        addPages: function(asset, data) {
            var pages = 0; var _this = this;
            $.each(data.general, function(index, value) { // suboptimal
                if (value.id === 'pages') pages = value.value;
            });
            if (!pages || pages < 2) return;
            var pagesTitle = $('<tr />').addClass('title pages').appendTo(this.table);
            $('<td />').html(this.strings['PdfDocumentProperties.pages']).appendTo(pagesTitle);
            $('<td />').html('<span class="anura-page-prev" title="key: &#9040;">'+_this.options.paginator_nav.prev+'</span><span class="anura-page-counter">1</span>/<span class="anura-page-total">'+pages+'</span>'+
                '<span class="anura-page-next" title="key: &#9047;">'+_this.options.paginator_nav.next+'</span>').appendTo(pagesTitle);
            var pagesContainer = $('<td />').attr('colspan', 2).appendTo($('<tr />').addClass('pages-container').appendTo(this.table));
            var pageScroller = $('<div />').addClass('pages-scroller').appendTo(pagesContainer);
            var pagesWrapper = $('<div />').addClass('pages-wrapper').appendTo(pageScroller);
            for (var i = 0; i < pages; i++) {
                $('<img />').attr('src', this.options.server+'/asset'+this.options.ext+'thumb='+asset+'&page='+i+($.anura.tokenProvider?'&token='+$.anura.tokenProvider():'')+'&img=a.jpg')
                    .attr('title', this.strings['paging.page']+' '+(i+1)).data('page', i).addClass(i == 0?'anura-active':'').appendTo(pagesWrapper);
            }
            pagesContainer.find('img').on('click', function() {
                var pageNr = $(this).data('page');
                pagesContainer.find('img.anura-active').removeClass('anura-active');
                $(_this.options.paginator_target).attr('src', _this.options.server+'/asset'+_this.options.ext+'preview='+asset+'&page='+pageNr+($.anura.tokenProvider?'&token='+$.anura.tokenProvider():'')+'&img=a.jpg');
                $('span.anura-page-counter').text((pageNr+1));
                $(this).addClass('anura-active');
                if (_this.options.scrollfix !== null) $(this).get(0).scrollIntoView(_this.options.scrollfix);
            });
            $('#colorbox').off('keydown').on('keydown', function(e) {
                if (e.keyCode === 38) var focus = pagesContainer.find('img.anura-active').prev().click();
                else if (e.keyCode === 40) var focus = pagesContainer.find('img.anura-active').next().click();
            });
            pagesTitle.find('.anura-page-prev').on('click', function() {pagesContainer.find('img.anura-active').prev().click()});
            pagesTitle.find('.anura-page-next').on('click', function() {pagesContainer.find('img.anura-active').next().click()});
            pageScroller.css('max-width', (this.container.width()-15)+'px'); // meh
        },
        addShare: function(asset, data) {
            $('<td />').attr('colspan', 2).html(this.strings['MENUITEMID_SHARE'] || 'Share').appendTo($('<tr />').addClass('title share').appendTo(this.table));
            // this is all a bit silly, but browsers only let you copy texts from a visible input field
            var targetId = 'anura-clipboard-target-'+asset;
            var holder = $('<td />').attr('colspan', 2).appendTo($('<tr />').addClass('clipboard').appendTo(this.table));
            var link = $('<a />').attr('rel', 'nofollow').attr('href', window.location).attr('targetId', '_blank')
                .attr('data-clipboard-target', '#'+targetId).addClass('anura-details-share anura-clipboard')
                .text(this.strings['common.copyToClipboard']).appendTo(holder);
            $('<input class="anura-clipboard-target" id="'+targetId+'"/>').val(window.location).prop('readonly', true).appendTo(holder);
            var setLinkState = function (e, link, clazz) {
                e.clearSelection();
                link.removeClass('anura-success anura-error').addClass(clazz);
                setTimeout(function () {link.removeClass('anura-success anura-error')}, 2500);
            };
            new ClipboardJS('.anura-details-share.anura-clipboard')
                .on('success', function(e) {setLinkState(e, link, 'anura-success');})
                .on('error', function(e) {setLinkState(e, link, 'anura-error');});
        },
        addDownloads: function(asset, data) {
            var _this = this;
            $('<td />').attr('colspan', 2).html(data.download_title).appendTo($('<tr />').addClass('title download').appendTo(this.table));
            if (this.options.basket) {
                var basket_holder = $('<td />').attr('colspan', 2).appendTo($('<tr />').addClass('basket').appendTo(this.table));
                this.createBasketLink(asset, basket_holder);
            }
            if (this.options.send) {
                var send_holder = $('<td />').attr('colspan', 2).appendTo($('<tr />').addClass('mailto').appendTo(this.table));
                var send_link = $('<a />').attr('rel', 'nofollow').addClass('anura-details-download mailto').appendTo(send_holder)
                    .attr('href', 'mailto:?body='+encodeURIComponent(_this.options.server+'/asset'+_this.options.ext
                        +'download='+asset+'&format='+this.options.send+'&locale='+_this.options.locale+(_this.options.force_zip?'&forcezip=true':'')));
                $($.anura.icons.mail).appendTo(send_link);
                $('<div />').addClass('anura-details-download-name').html(this.strings['MENUITEMID_MAILTO'])
                    .attr('title', this.strings['MENUITEMID_MAILTO']).appendTo(send_link);
            }
            if (this.options.download_relations) {
                var relations = 0;
                $.each(data.general, function(index, value) { // suboptimal
                    if (value.id === 'relations') relations = value.value;
                });
                if (relations > 0) {
                    var rel_holder = $('<td />').attr('colspan', 2).appendTo($('<tr />').addClass('relations').appendTo(this.table));
                    var rel_link = $('<a />').attr('rel', 'nofollow').attr('target', '_blank').addClass('anura-details-download relations').appendTo(rel_holder)
                        .attr('href', _this.options.server + '/asset' + _this.options.ext + 'relations=' + asset
                            + '&locale=' + _this.options.locale + (_this.options.force_zip ? '&forcezip=true' : '')
                            + ($.anura.tokenProvider ? '&token=' + $.anura.tokenProvider() : '')
                            + (this.options.downloadFormat ? '&format=' + this.options.downloadFormat : ''));
                    $($.anura.icons.download).appendTo(rel_link);
                    $('<div />').addClass('anura-details-download-name').html(this.strings['anura.relations']+' <span class="anura-relation-count">'+relations+'</span>')
                        .attr('title', this.strings['anura.relations']).appendTo(rel_link);
                }
            }
            var extension = undefined;
            $.each(data.general, function(index, value) { // suboptimal
                if (value.id === 'extension') extension = value.value;
            });
            if (data.downloads && data.downloads.length === 0) _this.table.find('.basket, .mailto, .title.download').remove();
            else if (this.options.download_directly) {
                var populateDlfs = function (downloadFormats) {
                    $.each(downloadFormats, function(index, format) {
                        if (format.extensions && extension && $.inArray(extension, format.extensions) === -1) return;
                        var link = $('<a />').addClass('anura-details-download').attr('targetId', '_blank').appendTo(
                            $('<td />').attr('colspan', 2).appendTo($('<tr />').addClass('dlf dlf'+format.id).appendTo(_this.table)));
                        var linkUrl = 'download='+asset+'&locale='+_this.options.locale+($.anura.tokenProvider?'&token='+$.anura.tokenProvider():'')+(_this.options.force_zip?'&forcezip=true':'');
                        if (format.features && $.inArray('editable', format.features) !== -1 && jQuery().anuraCropper) {
                            link.attr('href', 'javascript:').on('click', function () {
                                (_this.options.caller ? _this.options.caller : _this.element).anuraCropper({
                                    server: _this.options.server,
                                    locale: _this.options.locale,
                                    assetId: asset,
                                    downloadFormat: format.id
                                });
                            })
                        } else if (format.url) {
                            link.attr('href', _this.options.server+'/'+format.url+'?'+linkUrl).on('click', function () {$(window).trigger('anura-download', [asset, format.id]);});
                        } else {
                            link.attr('href', _this.options.server+'/asset'+_this.options.ext+'format='+format.id+'&'+linkUrl).on('click', function () {$(window).trigger('anura-download', [asset, format.id]);});
                        }
                        $('<img />').attr('src', _this.options.icon_transform(_this.options.server+'/'+format.icon)).appendTo(link);
                        var name = $('<div />').addClass('anura-details-download-name').appendTo(link);
                        if (format.description && _this.options.show_dlf_description) {
                            $('<span/>').addClass('anura-dlf-name').text(format.name).attr('title', format.name).appendTo(name);
                            $('<span/>').addClass('anura-dlf-descr').text(format.description).attr('title', format.description).appendTo(name);
                        } else {
                            name.text(format.name).attr('title', format.description);
                        }
                    });
                };
                if (data.downloads) {
                    populateDlfs(data.downloads);
                } else {
                    _this.requestDownloadFormats(populateDlfs)
                }
            }
        },
        addLinks: function(asset, data) {
            if (!data.links || data.links.length === 0) return;
            var _this = this;
            var td = $('<td />').attr('colspan', 2).html(data.links_title).appendTo($('<tr />').addClass('title links').appendTo(this.table));
            $('<a/>').addClass('anura-details-download-name').text(this.strings['search.selectAll']).appendTo(td)
                .on('click', function() {$('.anura-details-link-name .anura-details-download:not(\'.added\')').click()});
            data.links = data.links.sort(function(a, b) {return a.name > b.name});
            $.each(data.links, function(index, link) {
                var td = $('<td />').attr('colspan', 2).data('id', link.id).appendTo($('<tr />').addClass('link').appendTo(_this.table));
                $('<img/>').attr('src', _this.options.server + '/asset'+_this.options.ext+'thumb='+link.id+'&img=a.jpg')
                    .appendTo($('<div/>').addClass('anura-details-link-thmb').appendTo(td));
                var name = link.alt_name?link.alt_name:link.name;
                _this.createBasketLink(link.id, $('<div/>').addClass('anura-details-link-name').html(name+'<br>').attr('title', name).appendTo(td));
            });
        },
        addRelations: function(asset, data) {
            var _this = this;
            var relation_title = $('<tr />').addClass('title relations').appendTo(this.table);
            $('<td />').attr('colspan', 2).html(this.strings['anura.relations'] || 'Relations').appendTo(relation_title);
            var relation_row = $('<tr />').addClass('relations').appendTo(this.table);
            var relation_container = $('<td />').attr('colspan', 2).addClass('anura-details-relations').appendTo(relation_row);

            relation_container[_this.options.relations.view]($.extend(
                {
                    size: 'small', paginate: 8,
                    lightbox_rel: 'lightbox-anura-relations-'+asset,
                    detail_view: function(item, trigger) {_this.options.caller ? _this.options.caller.showAsset(trigger) : console.log('lightbox not found, please configure one');}
                }, _this.options.relations.view_settings, {
                    server: _this.options.server, locale: _this.options.locale,
                    detail_trigger: false /*otherwise it triggers itself*/,
                    relations: {id: asset, type: _this.options.relations.type}
                }
            )).on('anura-loaded-first', function(e, options, total) {
                if (total === 0) {
                    relation_title.remove();
                    relation_row.remove();
                }
            }).on('anura-error', function(e) {
                relation_title.remove();
                relation_row.remove();
            });
        },
        addSimilar: function(asset, data) {
            if ($.isEmptyObject(data.keyword_list)) return;
            var _this = this;
            var td = $('<td />').attr('colspan', 2).text(this.strings['anura.similarAssets']).appendTo($('<tr />').addClass('title similar').appendTo(this.table));
            $('<a/>').addClass('anura-details-download-name').text(this.strings['search.selectAll']).appendTo(td)
                .on('click', function() {$('.anura-details-similar-name .anura-details-download:not(\'.added\')').click()});
            var keywords = [], counter = 0, baseurl = _this.options.server + '/asset'+_this.options.ext;
            $.each(data.keyword_list, function(name, value) {keywords.push(value.id)});
            $('<td />').attr('colspan', 2).addClass('anura-similar').appendTo($('<tr />').appendTo(this.table));
            $.anura.getJSON(this, this.options.server+'/node'+this.options.ext+'callback=?', {
                search_keyword: keywords, search_node_or_mode: true, search_limit: 6
            }, 'anuraSimilar', function(data) {
                $.each(data, function(index, item) {
                    if (item.id === asset || ++counter > 5) return; // exclude self
                    var td = $('<td />').attr('colspan', 2).data('id', item.id).appendTo($('<tr />').addClass('similar').appendTo(_this.table));
                    $('<img/>').attr('src', baseurl+'thumb='+item.id+'&img=a.jpg').attr('rel', baseurl+'preview='+item.id+'&img=a.jpg')
                        .appendTo($('<div/>').addClass('anura-details-similar-thmb').appendTo(td));
                    var name = item.alt_name?item.alt_name:item.name;
                    _this.createBasketLink(item.id, $('<div/>').addClass('anura-details-similar-name').html(name+'<br>').attr('title', name).appendTo(td));
                });
            });
        },
        addVersions: function (asset, data) {
            var _this = this;
            if ($.isEmptyObject(data.versions) || data.versions.length < 2) return;
            $('<td />').attr('colspan', 2).text(this.strings['detailview.assetversion.title']).appendTo($('<tr />').addClass('title versions').appendTo(this.table));
            var rel = 'anura-versions-'+asset;
            $.each(data.versions, function (index, version) {
                var td = $('<td />').attr('colspan', 2).appendTo($('<tr />').addClass('version version-'+version.version+' version-'
                        +(version.active ? 'active' : 'inactive')).appendTo(_this.table));
                var container = $('<div>').addClass('version-container').appendTo(td);
                var thumb = $('<div>').addClass('version-thumb').css('background-image', 'url("'+version.urls.thumb+'")').appendTo(container);
                thumb.on('click', function () {
                    _this.options.version = version.version;
                    var lightbox_owner = _this.element.data('anura-lightbox-owner');
                    if (lightbox_owner) lightbox_owner.anuraLightbox({
                        action: {
                            title: version.name + ' ('+_this.strings['detailview.assetversion.version']+' '+version.version+')',
                            url: version.urls.largepreview && ($(window).width() > 1024 || $(window).height() > 1024) ? version.urls.largepreview : version.urls.preview
                        }});
                });
                var info = $('<div>').addClass('version-info').appendTo(container);
                $('<p>').addClass('version-name').text(version.name).attr('title', version.name).appendTo(info);
                var verp = $('<p>').addClass('version-number').text(_this.strings['detailview.assetversion.version']+': '+version.version
                        +(version.active?(' ('+_this.strings['detailview.assetversion.current']+')'):'')+' – ').appendTo(info);
                $('<a>').addClass('version-download').text(data.download_title).attr('href', _this.options.server+'/asset'+_this.options.ext+
                        'download='+asset+':'+_this.options.downloadFormat+'&version='+version.version+
                        '&locale='+_this.options.locale+($.anura.tokenProvider?'&token='+$.anura.tokenProvider():'')
                        +(_this.options.force_zip?'&forcezip=true':'')).appendTo(verp);
                $('<p>').addClass('version-date').text(_this.strings['detailview.assetversion.date']+': '+version.date).appendTo(info);
            });
        },

        addGetReviews: function () {
            var _this = this;
            var link = this.options.server+'/review'+this.options.ext+'&get='+this.options.asset;
            if($('.title.reviews').length === 0){
                $('<td />').attr('colspan', 2).text(_this.strings['assetReview.view.asset.title']).appendTo($('<tr />').addClass('title reviews').appendTo(_this.table));
            }
            var trShowReviewsHeader = $('<tr />').addClass('review-show-reviews-header').appendTo(_this.table);
            var tdShowReviewsHeader = $('<td />').attr('colspan', 2).appendTo(trShowReviewsHeader);
            var divShowReviewsHeader = $('<div id="review-headerReviewList">').appendTo(tdShowReviewsHeader);
            var button = $('<button id="review-showReviewsArrow">').appendTo(divShowReviewsHeader);
            $($.anura.icons.down).appendTo(button);
            $('<span>').text(_this.strings['assetReview.view.asset.headerReviewList']).appendTo(divShowReviewsHeader);
            var divReviewList = $('<div id="review-reviewsList">').appendTo(tdShowReviewsHeader);
            divShowReviewsHeader.on('click', function () {
                divShowReviewsHeader.toggleClass('anura-review-toggle');
                divReviewList.toggleClass('anura-review-toggle');
            });
            var averageRatingContainer = $('<div id="review-average-rating-container">').appendTo(divReviewList);
            $.anura.getJSON(this, link+'&callback=?', {}, 'getAnuraAssetReview'+this.options.asset, function(data) {
                var averageRating = data.averageRating || 0;
                _this.addStarsAndRating(averageRatingContainer, averageRating);
                _this.addAverageRating(averageRatingContainer);
                _this.setStarsAndRating(averageRatingContainer, averageRating);
                _this.setAverageRating(averageRatingContainer, averageRating);
                var title = $('<div id="review-average-rating-title">').appendTo(averageRatingContainer);
                $('<p id="review-average-rating-title-value">').text(_this.strings['assetReview.view.asset.label.ratingAverage']).appendTo(title);
                if ($.isEmptyObject(data.reviewList)) return;
                $.each(data.reviewList, function(index, review){
                    _this.addSingleCommentNode(review);
                });
            }, function(){
                console.log('failed to get reviews')
            });
        },

        addPostReviews: function(){
            var _this = this;
            if($('.title.reviews').length === 0) $('<td />').attr('colspan', 2).text(_this.strings['assetReview.view.asset.title']).appendTo($('<tr />').addClass('title reviews').appendTo(_this.table));
            var trAddReviewHeader = $('<tr />').addClass('review-add-review-header').appendTo(_this.table);
            var tdAddReviewHeader = $('<td />').attr('colspan', 2).appendTo(trAddReviewHeader);
            var divAddReviewHeader = $('<div id="review-headerNewReview">').appendTo(tdAddReviewHeader);
            var button = $('<button id="review-newReviewPlus">').appendTo(divAddReviewHeader);
            $($.anura.icons.plus).appendTo(button);
            $('<span>').text(_this.strings['assetReview.view.asset.headerNewReview']).appendTo(divAddReviewHeader);
            var trAddReviewContent = $('<tr />').addClass('review-add-content').appendTo(_this.table);
            var tdAddReviewContent = $('<td />').attr('colspan', 2).appendTo(trAddReviewContent);
            var divAddReviewContent = $('<div id="review-newReviewContent">').appendTo(tdAddReviewContent);
            divAddReviewHeader.on('click', function () {
                divAddReviewHeader.toggleClass('anura-review-toggle');
                divAddReviewContent.toggleClass('anura-review-toggle');
            });
            var divRating = $('<div id="review-add-rating-container">').appendTo(divAddReviewContent);
            $('<p id="review-add-rating-title">').text(_this.strings['assetReview.view.asset.label.rating']).appendTo(divRating);
            var rating = $('<div id="review-add-rating">').appendTo(divRating);
            for(var i = 5; i > 0; i--){
                $('<input type="radio" name="rating" value="' + i + '" id="' + i + '"><label for="' + i + '">☆</label>').appendTo(rating);
            }
            var divComment = $('<div id="review-add-comment-container">').appendTo(divAddReviewContent);
            $('<p id="review-add-comment-title">').text(_this.strings['assetReview.view.asset.label.comment']).appendTo(divComment);
            $('<textarea id="review-add-comment" name="text" rows="6">').appendTo(divComment);
            var divButtons = $('<div id="review-add-button-container">').appendTo(divAddReviewContent);
            _this.addReviewButtons(divButtons, function(){_this.saveReview()}, $.anura.icons.ok, _this.strings['common.actions.save']);
            _this.addReviewButtons(divButtons, function(){_this.clearReview()}, $.anura.icons.close, _this.strings['common.actions.clear']);
        },

        addSingleCommentNode: function(review) {
            var _this = this;
            var firstItem = $('.review-container').first();
            var newItem = $('<div>').addClass('review-container');
            var reviewList = $('#review-reviewsList');
            (firstItem.length > 0) ? newItem.insertBefore(firstItem) : newItem.appendTo(reviewList);
            var value = review.reviewId;
            $('<input type="text" />').addClass('review-commentId').val(value).hide().appendTo(newItem);
            var creatorContainer = $('<div>').addClass('review-creator-container').appendTo(newItem);
            $('<input type="text" />').addClass('review-creator-creatorId').val(review.creator.userId).hide().appendTo(creatorContainer);
            $('<div>').addClass('review-creator-avatar').html(review.creator.avatar).appendTo(creatorContainer);
            $('<div>').addClass('review-creator-creatorName').html(review.creator.userName).appendTo(creatorContainer);
            var ratingAndDate = $('<div>').addClass('review-rating-date-container').appendTo(newItem);
            _this.addStarsAndRating(ratingAndDate, review.rating);
            _this.setStarsAndRating(ratingAndDate, review.rating);
            var dateContainer = $('<div>').addClass('review-date').appendTo(ratingAndDate);
            var date = review.lastModifiedDate;
            if (date == null) date = review.createDate;
            date = new Date(date);
            $('<p>').addClass('review-date-value').text(date.toLocaleDateString()).attr('title', date.toLocaleString()).appendTo(dateContainer);
            $('<div>').addClass('review-comment').html(review.comment).appendTo(newItem);
        },

        addReviewButtons: function(divButtons, f, icon, text) {
            var button = $('<button>').addClass('review-button').appendTo(divButtons).on('click', f)
            var div = $('<div>').addClass('review-button-content').appendTo(button);
            $(icon).appendTo(div);
            $('<div>').text(text).appendTo(div);
        },

        saveReview: function() {
            var _this = this;
            var link = this.options.server+'/review'+this.options.ext+'&post='+this.options.asset;
            var comment = this.container.find('#review-add-comment').val();
            var ratingElement = $("input[name='rating']:checked");
            var rating = 0;
            if (ratingElement.length === 0 && comment === ''){
                alert(_this.strings['assetReview.view.asset.label.checkReviewError']);
                return;
            }
            var inputs = this.container.find('.review-add-content textarea, .review-add-content button').prop('disabled', true);
            if (ratingElement.length > 0) rating = ratingElement.val();
            $.anura.getJSON(this, link+'&callback=?', {comment: comment, rating: rating}, 'postAnuraAssetReview'+this.options.asset, function(data) {
                inputs.prop('disabled', false);
                if($.inArray('get_reviews', _this.options.show) !== -1){
                    _this.addSingleCommentNode(data.anuraReview);
                    _this.clearReview();
                    var container = _this.container.find('#review-average-rating-container');
                    _this.setStarsAndRating(container, data.averageRating);
                    _this.setAverageRating(container, data.averageRating);
                }
            }, function(){
                inputs.prop('disabled', false);
                alert("fail to upload the review");
            });
        },

        clearReview: function(){
            this.container.find('#review-add-comment').val('');
            this.container.find('input[name="rating"]:checked').prop('checked', false);
        },

        addStarsAndRating: function(container, rating){
            var starContainer = $('<div>').addClass('review-stars-container').appendTo(container);
            var backStarsContainer = $('<div>').addClass('review-back-stars-container').appendTo(starContainer);
            for(var i = 1; i <= 5; i++){
                $('<span>').text('☆').appendTo(backStarsContainer);
            }
            var frontStarsContainer = $('<div>').addClass('review-front-stars-container').appendTo(backStarsContainer);
            for(var i = 1; i <= 5; i++){
                $('<span>').text('★').appendTo(frontStarsContainer);
            }
            var ratingContainer = $('<div>').addClass('review-rating-container').appendTo(container);
            $('<input type="number" />').addClass('review-rating-value').val(rating).hide().appendTo(ratingContainer);
        },

        addAverageRating: function(container) {
            var ratingContainer = container.find('.review-rating-container');
            $('<p/>').addClass('review-rating-value-display').appendTo(ratingContainer);
        },

        setStarsAndRating: function(container, rating) {
            var frontStars = container.find('.review-front-stars-container');
            var percent = rating == null ? 0 : (rating / 5 * 100);
            frontStars.width(percent + '%');
            container.find('.review-rating-value').val(percent);
        },

        setAverageRating: function(container, rating) {
            container.find('.review-rating-value-display').text((rating ? rating : 0) + ' / 5');
        },

        addRow: function(name, value, tr_class, tooltip) {
            if (!value) return;
            var tr = $('<tr />').addClass(tr_class).appendTo(this.table);
            var name = $('<td />').addClass('anura-name').html(name).appendTo(tr);
            if (tooltip) name.attr('title', tooltip).addClass('anura-has-info');
            if (typeof value.replace !== "undefined") value = value.replace(/>\n</g, '><').replace(/(?:\r\n|\r|\n)/g, '<br />')
            var val = $('<td />').addClass('anura-value').html(value).appendTo(tr);
        },
        createBasketLink: function(asset, append_to) {
            if (!this.options.basket) return;
            var basket_link = $('<a />').addClass('anura-details-download').appendTo(append_to);
            $($.anura.icons.basket).appendTo(basket_link);
            var basket_item = $('<div />').addClass('anura-details-download-name').appendTo(basket_link);
            if ($.inArray(asset, this.options.basket.data('assets')) !== -1) {
                basket_link.addClass('added');
                basket_item.html(this.strings['anura.removeFromBasket']).attr('title', this.strings['anura.removeFromBasket']);
            } else {
                basket_item.html(this.strings['anura.addToBasket'] || 'Add to basket').attr('title', this.strings['anura.addToBasket']);
            }
            var _this = this;
            basket_link.on('click', function() {
                _this.options.basket.anuraBasket(basket_link.hasClass('added') ? {remove: asset} : {add: asset, force_zip: _this.options.force_zip});
            });
            this.options.basket.on('anura-asset-added', function (event, options, asset) {
                $('#basket'+asset).addClass('added').attr('title', _this.strings['anura.removeFromBasket']);
                basket_item.html(_this.strings['anura.removeFromBasket'] || 'Remove from basket').attr('title', _this.strings['anura.removeFromBasket']);
                basket_link.addClass('added');
            }).on('anura-asset-removed anura-asset-failed anura-cleared', function () {
                $('#basket'+asset).removeClass('added').attr('title', _this.strings['anura.addToBasket']);
                basket_item.html(_this.strings['anura.addToBasket'] || 'Add to basket').attr('title', _this.strings['anura.addToBasket']);
                basket_link.removeClass('added');
            });
            return basket_link;
        },
        requestDownloadFormats: function(callback) {
            var _this = this;
            var onComplete = function(data) {
                if (data) {
                    window.anura_download_formats = data;
                    if (_this.options.send === true) _this.options.send = data[0].id;
                }
                if (callback) callback(data);
            };
            if (window.anura_download_formats) { // share globally
                onComplete(window.anura_download_formats);
            } else {
                $.anura.getJSON(this, this.options.server+'/asset'+this.options.ext+'callback=?',
                    {downloadformats: 1, locale: this.options.locale}, 'anuraDetailsDlf', onComplete,
                    function() {if (callback) callback();});
            }
        },
        isNodeClickable: function(id) {
            return id && $('#anura-node-'+id+', .anura-search-noderef option[value="'+id+'"]').not('.anura-facet-none').length > 0;
        },
        asMaybeClickableLink: function(node) {
            var isClickable = this.isNodeClickable(node.id);
            var keyword = $(isClickable?'<a/>':'<span/>').data('id', node.id).text(node.name);
            if (node.path) keyword.attr('title', node.path+node.name);
            if (isClickable) keyword.on('click', this.options.noderef_callback);
            return keyword;
        }
    };
    $.fn[pluginName] = function ( options ) {
        return this.each(function (i, item) { // iterate matches
            var instance = $.data(item, "plugin_" + pluginName);
            if (!instance) { // if it has no instance yet, create it
                $.data(item, "plugin_" + pluginName, new Plugin( item, options ));
            } else { // otherwise merge new options with existing ones, empty and re-init
                instance.options = $.extend( {}, instance.options, options );
                instance.init();
            }
        });
    };
})( jQuery, window, document );
